"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commandline_1 = require("../../util/commandline");
const apis_1 = require("../../util/apis");
const interaction_1 = require("../../util/interaction");
const profile_1 = require("../../util/profile");
let SetCurrentAppCommand = class SetCurrentAppCommand extends commandline_1.Command {
    constructor(args) {
        super(args);
    }
    run(client) {
        return __awaiter(this, void 0, void 0, function* () {
            const newDefault = profile_1.toDefaultApp(this.appId);
            if (!newDefault) {
                return commandline_1.failure(commandline_1.ErrorCodes.InvalidParameter, `'${this.appId}' is not a valid application.`);
            }
            const apps = yield interaction_1.out.progress("Reading available apps...", apis_1.clientCall((cb) => client.appsOperations.list(cb)));
            const found = apps.find((app) => app.name === newDefault.appName && app.owner.name === newDefault.ownerName);
            if (!found) {
                return commandline_1.failure(commandline_1.ErrorCodes.InvalidParameter, `You either do not have access to '${this.appId}' or there is no such application.`);
            }
            const profile = profile_1.getUser();
            if (!profile) {
                return commandline_1.failure(commandline_1.ErrorCodes.InvalidParameter, `Could not find a logged in profile, please note that this command is not compatible with the '--token' parameter or the token environment variable. Use environment variable 'MOBILE_CENTER_CURRENT_APP' to set the default app instead.`);
            }
            profile.defaultApp = newDefault;
            profile.save();
            return commandline_1.success();
        });
    }
};
__decorate([
    commandline_1.name("app"),
    commandline_1.position(0),
    commandline_1.help("Owner/app to set as default"),
    commandline_1.required
], SetCurrentAppCommand.prototype, "appId", void 0);
SetCurrentAppCommand = __decorate([
    commandline_1.help("Set default application for all CLI commands. Not compatible when authenticating with '--token' or an environment variable. Use environment variable 'MOBILE_CENTER_CURRENT_APP' to set the default app instead.")
], SetCurrentAppCommand);
exports.default = SetCurrentAppCommand;
