"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commandline_1 = require("../../util/commandline");
const apis_1 = require("../../util/apis");
const interaction_1 = require("../../util/interaction");
const util_1 = require("util");
const PortalHelper = require("../../util/portal/portal-helper");
const debug = require("debug")("appcenter-cli:commands:build:queue");
let QueueBuildCommand = class QueueBuildCommand extends commandline_1.AppCommand {
    run(client, portalBaseUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const app = this.app;
            debug(`Queuing build for branch ${this.branchName}`);
            let queueBuildRequestResponse;
            try {
                queueBuildRequestResponse = yield interaction_1.out.progress(`Queueing build for branch ${this.branchName}...`, apis_1.clientRequest((cb) => client.builds.create(this.branchName, app.ownerName, app.appName, {
                    debug: this.debugLogs,
                    sourceVersion: this.sourceVersion,
                }, cb)));
            }
            catch (error) {
                if (error.statusCode === 400) {
                    return commandline_1.failure(commandline_1.ErrorCodes.IllegalCommand, `app ${app.appName} is not configured for building`);
                }
                else {
                    debug(`Request failed - ${util_1.inspect(error)}`);
                    return commandline_1.failure(commandline_1.ErrorCodes.Exception, "failed to queue build request");
                }
            }
            const buildId = queueBuildRequestResponse.result.id;
            const realBranchName = queueBuildRequestResponse.result.sourceBranch;
            const url = PortalHelper.getPortalBuildLink(portalBaseUrl, app.ownerName, app.appName, realBranchName, buildId.toString());
            interaction_1.out.report([
                ["Build ID", "buildId"],
                ["Build URL", "url"],
            ], { buildId, url });
            return commandline_1.success();
        });
    }
};
__decorate([
    commandline_1.help("Branch to be built"),
    commandline_1.shortName("b"),
    commandline_1.longName("branch"),
    commandline_1.required,
    commandline_1.hasArg
], QueueBuildCommand.prototype, "branchName", void 0);
__decorate([
    commandline_1.help("Enable debug mode"),
    commandline_1.shortName("d"),
    commandline_1.longName("debug-logs")
], QueueBuildCommand.prototype, "debugLogs", void 0);
__decorate([
    commandline_1.help("Source control version reference"),
    commandline_1.shortName("s"),
    commandline_1.longName("source-version"),
    commandline_1.hasArg
], QueueBuildCommand.prototype, "sourceVersion", void 0);
QueueBuildCommand = __decorate([
    commandline_1.help("Queue a new build")
], QueueBuildCommand);
exports.default = QueueBuildCommand;
