"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commandline_1 = require("../../../util/commandline");
const interaction_1 = require("../../../util/interaction");
const util_1 = require("util");
const apis_1 = require("../../../util/apis");
const date_helper_1 = require("./lib/date-helper");
const misc_1 = require("../../../util/misc");
const chalk = require("chalk");
const debug = require("debug")("appcenter-cli:commands:codepush:deployments:history");
let CodePushDeploymentHistoryCommand = class CodePushDeploymentHistoryCommand extends commandline_1.AppCommand {
    constructor(args) {
        super(args);
    }
    run(client) {
        return __awaiter(this, void 0, void 0, function* () {
            const app = this.app;
            let releases;
            let metrics;
            try {
                const releasesHttpRequest = yield interaction_1.out.progress("Getting CodePush releases...", apis_1.clientRequest((cb) => client.codePushDeploymentReleases.get(this.deploymentName, app.ownerName, app.appName, cb)));
                releases = releasesHttpRequest.result;
                const metricsHttpRequest = yield interaction_1.out.progress("Getting CodePush releases metrics...", apis_1.clientRequest((cb) => client.codePushDeploymentMetrics.get(this.deploymentName, app.ownerName, app.appName, cb)));
                metrics = metricsHttpRequest.result;
                const releasesTotalActive = metrics.reduce((sum, releaseMetrics) => (sum += releaseMetrics.active), 0);
                let tableTitles = ["Label", "Release Time", "App Version", "Mandatory", "Description", "Install Metrics"];
                tableTitles = tableTitles.map((title) => chalk.cyan(title));
                interaction_1.out.table(interaction_1.out.getCommandOutputTableOptions(tableTitles), releases.map((release) => {
                    let releaseRow = [
                        release.label,
                        date_helper_1.formatDate(release.uploadTime) + this.generateReleaseAdditionalInfoString(release),
                        release.targetBinaryRange,
                        release.isMandatory ? "Yes" : "No",
                        release.description,
                        this.generateReleaseMetricsString(release, metrics, releasesTotalActive),
                    ];
                    if (release.isDisabled) {
                        releaseRow = releaseRow.map((element) => this.applyDimChalkSkippingLineBreaks(element));
                    }
                    return releaseRow;
                }));
                return commandline_1.success();
            }
            catch (error) {
                debug(`Failed to get list of CodePush deployments - ${util_1.inspect(error)}`);
                if (error.statusCode === 404 && error.response.body === `Deployment "${this.deploymentName}" does not exist.`) {
                    const deploymentNotExistErrorMsg = `The deployment ${chalk.bold(this.deploymentName)} does not exist.`;
                    return commandline_1.failure(commandline_1.ErrorCodes.Exception, deploymentNotExistErrorMsg);
                }
                else if (error.statusCode === 404) {
                    const appNotFoundErrorMsg = `The app ${this.identifier} does not exist. Please double check the name, and provide it in the form owner/appname. \nRun the command ${chalk.bold(`${misc_1.scriptName} apps list`)} to see what apps you have access to.`;
                    return commandline_1.failure(commandline_1.ErrorCodes.NotFound, appNotFoundErrorMsg);
                }
                else {
                    return commandline_1.failure(commandline_1.ErrorCodes.Exception, error.response.body);
                }
            }
        });
    }
    generateReleaseAdditionalInfoString(release) {
        let additionalInfo = "";
        if (release.releaseMethod === "Promote") {
            additionalInfo = `(Promoted ${release.originalLabel} from ${release.originalDeployment})`;
        }
        else if (release.releaseMethod === "Rollback") {
            const labelNumber = parseInt(release.label.substring(1), 10);
            const previousReleaseLabel = "v" + (labelNumber - 1);
            additionalInfo = `(Rolled back ${previousReleaseLabel} to ${release.originalLabel})`;
        }
        return additionalInfo ? "\n" + chalk.magenta(additionalInfo) : "";
    }
    generateReleaseMetricsString(release, metrics, releasesTotalActive) {
        let metricsString = "";
        const releaseMetrics = metrics.find((metric) => metric.label === release.label);
        if (releaseMetrics) {
            const activePercent = releasesTotalActive ? (releaseMetrics.active / releasesTotalActive) * 100 : 0.0;
            let percentString;
            if (activePercent === 100.0) {
                percentString = "100%";
            }
            else if (activePercent === 0.0) {
                percentString = "0%";
            }
            else {
                percentString = activePercent.toPrecision(2) + "%";
            }
            metricsString += chalk.green("Active: ") + percentString + ` (${releaseMetrics.active} of ${releasesTotalActive})`;
            if (releaseMetrics.installed != null) {
                metricsString += "\n" + chalk.green("Installed: ") + releaseMetrics.installed;
                const pending = releaseMetrics.downloaded - releaseMetrics.installed - releaseMetrics.failed;
                if (pending > 0) {
                    metricsString += ` (${pending} pending)`;
                }
            }
            if (releaseMetrics.failed > 0) {
                metricsString += "\n" + chalk.green("Rollbacks: ") + chalk.red(releaseMetrics.failed.toString());
            }
        }
        else {
            metricsString = chalk.magenta("No installs recorded");
        }
        if (release.rollout != null && release.rollout !== 100) {
            metricsString += "\n" + chalk.green("Rollout: ") + release.rollout + "%";
        }
        if (release.isDisabled) {
            metricsString += "\n" + chalk.green("Disabled: ") + "Yes";
        }
        return metricsString;
    }
    applyDimChalkSkippingLineBreaks(applyString) {
        // Used to prevent "chalk" from applying styles to linebreaks which
        // causes table border chars to have the style applied as well.
        let chalkedString = "";
        if (applyString) {
            chalkedString = applyString
                .split("\n")
                .map((line) => chalk.dim(line))
                .join("\n");
        }
        return chalkedString;
    }
};
__decorate([
    commandline_1.help("Specifies CodePush deployment name to view history"),
    commandline_1.required,
    commandline_1.name("deployment-name"),
    commandline_1.position(0)
], CodePushDeploymentHistoryCommand.prototype, "deploymentName", void 0);
CodePushDeploymentHistoryCommand = __decorate([
    commandline_1.help("Display the release history for a CodePush deployment")
], CodePushDeploymentHistoryCommand);
exports.default = CodePushDeploymentHistoryCommand;
