"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commandline_1 = require("../../../util/commandline");
const apis_1 = require("../../../util/apis");
const interaction_1 = require("../../../util/interaction");
const util_1 = require("util");
const _ = require("lodash");
const pLimit = require("p-limit");
const debug = require("debug")("appcenter-cli:commands:distribute:groups:list");
let ListDistributionGroupsCommand = class ListDistributionGroupsCommand extends commandline_1.AppCommand {
    run(client) {
        return __awaiter(this, void 0, void 0, function* () {
            const app = this.app;
            debug("Getting list of the distribution groups");
            let distributionGroupsListRequestResponse;
            try {
                distributionGroupsListRequestResponse = yield interaction_1.out.progress("Getting list of the distribution groups...", apis_1.clientRequest((cb) => client.distributionGroups.list(app.ownerName, app.appName, cb)));
            }
            catch (error) {
                debug(`Failed to get list of the distribution groups - ${util_1.inspect(error)}`);
                return commandline_1.failure(commandline_1.ErrorCodes.Exception, "failed to fetch list of all distribution groups");
            }
            if (distributionGroupsListRequestResponse.response.statusCode >= 400) {
                return commandline_1.failure(commandline_1.ErrorCodes.Exception, "failed to fetch list of all distribution groups");
            }
            const distributionGroupsNames = _(distributionGroupsListRequestResponse.result)
                .sortBy((distributionGroup) => distributionGroup.name)
                .map((distributionGroup) => distributionGroup.name)
                .value();
            const limit = pLimit(10);
            debug("Creating requests for retrieving user counts of distribution groups");
            const distributionGroupUsersPromises = [];
            for (const distributionGroupName of distributionGroupsNames) {
                distributionGroupUsersPromises.push(limit(() => apis_1.clientRequest((cb) => client.distributionGroups.listUsers(this.app.ownerName, this.app.appName, distributionGroupName, cb))));
            }
            // Showing progress spinner while requests are being sent
            const requestsCompletedPromise = interaction_1.out.progress("Getting number of users for distribution groups", Promise.all(distributionGroupUsersPromises.map((dg) => dg.catch((res) => Promise.resolve()))));
            const userCounts = [];
            for (let i = 0; i < distributionGroupUsersPromises.length; i++) {
                const distributionGroupUsers = distributionGroupUsersPromises[i];
                let userCount;
                try {
                    debug(`Waiting for ${distributionGroupsNames[i]} distribution group users request response`);
                    const distributionGroupUsersRequestResponse = yield distributionGroupUsers;
                    if (distributionGroupUsersRequestResponse.response.statusCode >= 400) {
                        throw distributionGroupUsersRequestResponse.response.statusCode;
                    }
                    debug(`Request for the list of ${distributionGroupsNames[i]} distribution group users has succeeded`);
                    userCount = distributionGroupUsersRequestResponse.result.length.toString();
                }
                catch (error) {
                    debug(`Request for the list of ${distributionGroupsNames[i]} distribution group users has failed - ${util_1.inspect(error)}`);
                    userCount = "failed to get number of users";
                }
                userCounts.push(userCount);
            }
            // Waiting for spinner to stop
            yield requestsCompletedPromise;
            const outputArray = _.zip(distributionGroupsNames, userCounts);
            // Printing the result table
            interaction_1.out.table(interaction_1.out.getCommandOutputTableOptions(["Group", "Users"]), outputArray);
            return commandline_1.success();
        });
    }
};
ListDistributionGroupsCommand = __decorate([
    commandline_1.help("Lists all distribution groups of the app")
], ListDistributionGroupsCommand);
exports.default = ListDistributionGroupsCommand;
