"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commandline_1 = require("../../../util/commandline");
const apis_1 = require("../../../util/apis");
const interaction_1 = require("../../../util/interaction");
const util_1 = require("util");
const debug = require("debug")("appcenter-cli:commands:distribute:releases:delete");
let DeleteReleaseCommand = class DeleteReleaseCommand extends commandline_1.AppCommand {
    run(client) {
        return __awaiter(this, void 0, void 0, function* () {
            const app = this.app;
            const releaseId = Number(this.releaseId);
            if (!Number.isSafeInteger(releaseId) || releaseId <= 0) {
                return commandline_1.failure(commandline_1.ErrorCodes.InvalidParameter, `${this.releaseId} is not a valid release id`);
            }
            if (!(yield interaction_1.prompt.confirm(`Do you really want to delete release ${this.releaseId}?`))) {
                interaction_1.out.text(`Deletion of release ${this.releaseId} was cancelled`);
                return commandline_1.success();
            }
            let releaseDetails;
            try {
                debug("Loading release details");
                const httpRequest = yield interaction_1.out.progress("Loading release details...", apis_1.clientRequest((cb) => client.releasesOperations.getLatestByUser(this.releaseId, app.ownerName, app.appName, cb)));
                if (httpRequest.response.statusCode >= 400) {
                    throw httpRequest.response.statusCode;
                }
                else {
                    releaseDetails = httpRequest.result;
                }
            }
            catch (error) {
                if (error === 404) {
                    return commandline_1.failure(commandline_1.ErrorCodes.InvalidParameter, `release ${this.releaseId} doesn't exist`);
                }
                else {
                    debug(`Failed to load release details - ${util_1.inspect(error)}`);
                    return commandline_1.failure(commandline_1.ErrorCodes.Exception, "failed to load release details");
                }
            }
            try {
                debug("Removing release");
                const httpResponse = yield interaction_1.out.progress(`Removing the release...`, apis_1.clientRequest((cb) => client.releasesOperations.deleteMethod(releaseId, app.ownerName, app.appName, cb)));
                if (httpResponse.response.statusCode >= 400) {
                    throw httpResponse.result;
                }
            }
            catch (error) {
                if (error.code === "partially_deleted") {
                    return commandline_1.failure(commandline_1.ErrorCodes.Exception, `release ${this.releaseId} was removed from all distribution groups, but couldn't be deleted from AppCenter`);
                }
                else {
                    debug(`Failed to remove the release - ${util_1.inspect(error)}`);
                    return commandline_1.failure(commandline_1.ErrorCodes.Exception, `failed to delete the release`);
                }
            }
            interaction_1.out.text(`Release ${releaseDetails.shortVersion} (${releaseDetails.version}) with id: ${this.releaseId} has been deleted`);
            return commandline_1.success();
        });
    }
};
__decorate([
    commandline_1.help("Release ID"),
    commandline_1.shortName("r"),
    commandline_1.longName("release-id"),
    commandline_1.required,
    commandline_1.hasArg
], DeleteReleaseCommand.prototype, "releaseId", void 0);
DeleteReleaseCommand = __decorate([
    commandline_1.help("Deletes the release")
], DeleteReleaseCommand);
exports.default = DeleteReleaseCommand;
