"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commandline_1 = require("../../../util/commandline");
const apis_1 = require("../../../util/apis");
const interaction_1 = require("../../../util/interaction");
const util_1 = require("util");
const _ = require("lodash");
const debug = require("debug")("appcenter-cli:commands:distribute:stores:list");
let ListStoresCommand = class ListStoresCommand extends commandline_1.AppCommand {
    run(client) {
        return __awaiter(this, void 0, void 0, function* () {
            const app = this.app;
            debug("Getting list of the stores");
            let storesListRequestResponse;
            try {
                storesListRequestResponse = yield interaction_1.out.progress("Getting list of the stores...", apis_1.clientRequest((cb) => client.stores.list(app.ownerName, app.appName, cb)));
            }
            catch (error) {
                debug(`Failed to get list of the stores - ${util_1.inspect(error)}`);
                return commandline_1.failure(commandline_1.ErrorCodes.Exception, "failed to fetch list of all stores");
            }
            if (storesListRequestResponse.response.statusCode >= 400) {
                return commandline_1.failure(commandline_1.ErrorCodes.Exception, "failed to fetch list of all stores");
            }
            const sortedStores = _(storesListRequestResponse.result).sortBy((store) => [store.type, store.track, store.name]);
            const storesNames = sortedStores.map((store) => store.name).value();
            const storesTypes = sortedStores.map((store) => store.type).value();
            const storesTracks = sortedStores.map((store) => store.track).value();
            const outputArray = _.zip(storesNames, storesTypes, storesTracks);
            if (!outputArray.length) {
                interaction_1.out.text(`No stores configured for app '${app.ownerName}/${app.appName}'`);
                interaction_1.out.table(interaction_1.out.getCommandOutputTableOptions([]), outputArray);
            }
            else {
                interaction_1.out.table(interaction_1.out.getCommandOutputTableOptions(["Store", "Type", "Track"]), outputArray);
            }
            return commandline_1.success();
        });
    }
};
ListStoresCommand = __decorate([
    commandline_1.help("Lists all stores of the app")
], ListStoresCommand);
exports.default = ListStoresCommand;
