"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commandline_1 = require("../../util/commandline");
const interaction_1 = require("../../util/interaction");
const apis_1 = require("../../util/apis");
const debug = require("debug")("appcenter-cli:commands:orgs:show");
const util_1 = require("util");
const portal_helper_1 = require("../../util/portal/portal-helper");
const org_users_helper_1 = require("./lib/org-users-helper");
let OrgShowCommand = class OrgShowCommand extends commandline_1.Command {
    run(client, portalBaseUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const [users, apps, organizationDetails] = yield interaction_1.out.progress("Loading organization information...", Promise.all([org_users_helper_1.getOrgUsers(client, this.name, debug), this.getOrgApps(client, this.name), this.getOrgDetails(client, this.name)]));
            const admins = org_users_helper_1.pickAdmins(users);
            interaction_1.out.report([
                ["Display name", "displayName"],
                ["URL", "url"],
                ["Admins", "admins", (adminsArray) => adminsArray.map((admin) => admin.name).join(", ")],
                ["Apps", "appsCount"],
                ["Collaborators", "collaboratorsCount"],
            ], {
                displayName: organizationDetails.displayName,
                url: portal_helper_1.getPortalOrgLink(portalBaseUrl, this.name),
                admins,
                appsCount: apps.length,
                collaboratorsCount: users.length,
                origin: organizationDetails.origin,
            });
            return commandline_1.success();
        });
    }
    getOrgApps(client, organization) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const httpResponse = yield apis_1.clientRequest((cb) => client.appsOperations.listForOrg(organization, cb));
                if (httpResponse.response.statusCode < 400) {
                    return httpResponse.result;
                }
                else {
                    throw httpResponse.response;
                }
            }
            catch (error) {
                if (error.statusCode === 404) {
                    throw commandline_1.failure(commandline_1.ErrorCodes.InvalidParameter, `organization ${organization} doesn't exist`);
                }
                else {
                    debug(`Failed to load list of organization apps - ${util_1.inspect(error)}`);
                    throw commandline_1.failure(commandline_1.ErrorCodes.Exception, "failed to load list of organization apps");
                }
            }
        });
    }
    getOrgDetails(client, organization) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const httpResponse = yield apis_1.clientRequest((cb) => client.organizations.get(organization, cb));
                if (httpResponse.response.statusCode < 400) {
                    return httpResponse.result;
                }
                else {
                    throw httpResponse.response;
                }
            }
            catch (error) {
                if (error.statusCode === 404) {
                    throw commandline_1.failure(commandline_1.ErrorCodes.InvalidParameter, `organization ${organization} doesn't exist`);
                }
                else {
                    debug(`Failed to get organization details - ${util_1.inspect(error)}`);
                    throw commandline_1.failure(commandline_1.ErrorCodes.Exception, "failed to get organization details");
                }
            }
        });
    }
};
__decorate([
    commandline_1.help("Name of the organization"),
    commandline_1.shortName("n"),
    commandline_1.longName("name"),
    commandline_1.required,
    commandline_1.hasArg
], OrgShowCommand.prototype, "name", void 0);
OrgShowCommand = __decorate([
    commandline_1.help("Show information about organization")
], OrgShowCommand);
exports.default = OrgShowCommand;
