"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commandline_1 = require("../../util/commandline");
const interaction_1 = require("../../util/interaction");
const apis_1 = require("../../util/apis");
const debug = require("debug")("appcenter-cli:commands:orgs:update");
const util_1 = require("util");
let OrgUpdateCommand = class OrgUpdateCommand extends commandline_1.Command {
    run(client) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.newDisplayName == null && this.newName == null) {
                return commandline_1.failure(commandline_1.ErrorCodes.InvalidParameter, "nothing to update");
            }
            try {
                const httpContent = yield interaction_1.out.progress("Updating organization...", apis_1.clientRequest((cb) => client.organizations.update(this.name, {
                    displayName: this.newDisplayName,
                    name: this.newName,
                }, cb)));
                if (httpContent.response.statusCode < 400) {
                    if (this.newDisplayName) {
                        interaction_1.out.text(`Successfully changed display name of ${this.name} to ${this.newDisplayName}`);
                    }
                    if (this.newName) {
                        interaction_1.out.text(`Successfully renamed ${this.name} to ${this.newName}`);
                    }
                }
                else {
                    throw httpContent.response;
                }
            }
            catch (error) {
                switch (error.statusCode) {
                    case 404:
                        return commandline_1.failure(commandline_1.ErrorCodes.InvalidParameter, `organization ${this.name} doesn't exist`);
                    case 409:
                        return commandline_1.failure(commandline_1.ErrorCodes.InvalidParameter, `organization ${this.newName} already exists`);
                    default:
                        debug(`Failed to update organization - ${util_1.inspect(error)}`);
                        return commandline_1.failure(commandline_1.ErrorCodes.Exception, `failed to update organization`);
                }
            }
            return commandline_1.success();
        });
    }
};
__decorate([
    commandline_1.help("Name of the organization"),
    commandline_1.shortName("n"),
    commandline_1.longName("name"),
    commandline_1.required,
    commandline_1.hasArg
], OrgUpdateCommand.prototype, "name", void 0);
__decorate([
    commandline_1.help("New organization display name"),
    commandline_1.shortName("d"),
    commandline_1.longName("new-display-name"),
    commandline_1.hasArg
], OrgUpdateCommand.prototype, "newDisplayName", void 0);
__decorate([
    commandline_1.help("New organization name"),
    commandline_1.shortName("r"),
    commandline_1.longName("new-name"),
    commandline_1.hasArg
], OrgUpdateCommand.prototype, "newName", void 0);
OrgUpdateCommand = __decorate([
    commandline_1.help("Update organization information")
], OrgUpdateCommand);
exports.default = OrgUpdateCommand;
