"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDSymFile = void 0;
const test_manifest_1 = require("./test-manifest");
const glob = require("glob");
const path = require("path");
function getDSymFile(dSymDir) {
    return __awaiter(this, void 0, void 0, function* () {
        if (path.extname(dSymDir) !== ".dSYM") {
            throw new Error("Invalid dSYM directory: name of the directory must have extension *.dSYM");
        }
        const pattern = path.join(dSymDir, "Contents", "Resources", "DWARF", "*");
        const files = yield new Promise((resolve, reject) => {
            glob(pattern, (err, matches) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(matches);
                }
            });
        });
        if (files.length === 0) {
            throw new Error(`Invalid dSYM directory: cannot find any symbol file (${pattern})`);
        }
        else if (files.length > 1) {
            throw new Error(`Invalid dSYM directory: found more than one symbol file (${pattern})`);
        }
        else {
            const fullPath = files[0].replace(/\//g, path.sep);
            return yield test_manifest_1.TestRunFile.create(fullPath, path.basename(fullPath), "dsym-file");
        }
    });
}
exports.getDSymFile = getDSymFile;
