"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Messages = void 0;
const os = require("os");
var Messages;
(function (Messages) {
    let TestCloud;
    (function (TestCloud) {
        let Commands;
        (function (Commands) {
            Commands.Wizard = "Start a test run interactively. All the parameters will be prompted on-the-go.";
            Commands.GenerateAppium = "Generates an Appium project";
            Commands.GenerateUITest = "Generates a Xamarin.UITest project";
            Commands.PrepareAppium = "Creates an artifacts directory with Appium tests.";
            Commands.PrepareCalabash = `Creates an artifacts directory with Calabash tests.${os.EOL}` +
                `Required external tools:${os.EOL}` +
                `- Ruby ${os.EOL}` +
                `- Gem xamarin-test-cloud`;
            Commands.PrepareEspresso = "Creates an artifacts directory with Espresso tests.";
            Commands.PrepareUITests = `Creates an artifacts directory with Xamarin UI Tests.${os.EOL}` +
                `Required external tools:${os.EOL}` +
                `- .NET Framework on Windows, Mono Runtime on OS X${os.EOL}` +
                `- NuGet package Xamarin.UITests, version 2.0.1 or higher`;
            Commands.PrepareXCUITest = "Creates an artifacts directory with XCUITest tests.";
            Commands.RunAppium = "Starts a test run with Appium tests.";
            Commands.RunCalabash = `Starts a test run with Calabash tests.${os.EOL}` +
                `Required external tools:${os.EOL}` +
                `- Ruby ${os.EOL}` +
                `- Gem xamarin-test-cloud`;
            Commands.RunEspresso = "Starts a test run with Espresso tests.";
            Commands.RunManifest = "Starts a test run with previously prepared artifacts.";
            Commands.RunUITests = `Starts a test run with Xamarin UI Tests.${os.EOL}` +
                `Required external tools:${os.EOL}` +
                `- .NET Framework on Windows, Mono Runtime on OS X${os.EOL}` +
                `- NuGet package Xamarin.UITests, version 2.0.1 or higher`;
            Commands.RunXCUITest = "Starts a test run with XCUITest tests.";
            Commands.Status = "Checks the status of the started test run.";
            Commands.Stop = "Stop the started test run.";
            Commands.Download = "Download the report artifacts, unpack and merge them. This command is only available for UITest and Appium test runs.";
        })(Commands = TestCloud.Commands || (TestCloud.Commands = {}));
        let Arguments;
        (function (Arguments) {
            Arguments.Include = 'Additional files and directories to include. The value must be either path relative to the input directory, or be in format "targetDir=sourceDir"';
            Arguments.TestParameter = 'Additional test parameters. The value must be in format "key=value"';
            Arguments.AppPath = "Path to an application file";
            Arguments.AppPlatform = "The app's platform ('ios' or 'android')";
            Arguments.GenerateOutputPath = "The path where the tests will be generated";
            Arguments.PrepareArtifactsDir = "Path to the artifacts directory to create";
            Arguments.RunDevices = "Device selection slug";
            Arguments.RunDSymDir = "Path to the directory with iOS symbol files. This option is deprecated and ignored";
            Arguments.RunLocale = "The system locale for the test run. For example, en_US";
            Arguments.RunLanguage = "Override the language (iOS only) for the test run";
            Arguments.Fixture = "NUnit fixture / namespace to run. If used with include- / exclude-category the fixture filter is applied first (Can be used multiple times)";
            Arguments.IncludeCategory = "NUnit category to run. (Can be used multiple times)";
            Arguments.ExcludeCategory = "NUnit category to not run. (Can be used multiple times) (When include and/or fixture are combined with exclude, all tests with the included categories are run, except for those also marked with the excluded categories)";
            Arguments.TestChunk = "Run tests in parallel by method";
            Arguments.FixtureChunk = "Run tests in parallel by fixture, ignore the use of include- / exclude-category";
            Arguments.RunTestSeries = "Name of the test series";
            Arguments.RunAsync = "Exit the command when tests are uploaded, without waiting for test results";
            Arguments.Timeout = "Maximum time (in seconds) to wait for test results";
            Arguments.AppiumBuildDir = "Path to the directory with the Appium tests (usually <project>/target/upload)";
            Arguments.CalabashProjectDir = "Path to the Calabash workspace directory (usually <project>/features)";
            Arguments.CalabashSignInfo = "Use Signing Info for signing the test server";
            Arguments.CalabashConfigPath = "Path to the Cucumber configuration file (usually cucumber.yml)";
            Arguments.CalabashProfile = "Profile to run. It must exist in the configuration file";
            Arguments.CalabashSkipConfigCheck = "Force running without Cucumber profile";
            Arguments.EspressoBuildDir = "Path to the Espresso output directory (usually <project>/build/outputs/apk)";
            Arguments.EspressoTestApkPath = "Path to the *.apk file with the Espresso tests. If not set, build-dir is used to discover it";
            Arguments.UITestsBuildDir = "Path to the directory with the built test assemblies (usually <project>/bin/<configuration>)";
            Arguments.UITestsStoreFilePath = "Path to the keystore file";
            Arguments.UITestsStorePassword = 'Password to the keystore. Corresponds to the "-storepass" argument in jarsigner';
            Arguments.UITestsKeyAlias = 'Alias to the key in the keystore. Corresponds to the "-alias" argument in jarsigner';
            Arguments.UITestsKeyPassword = 'Password to the matching private key in the keystore. Corresponds to the "-keypass" argument in jarsigner';
            Arguments.UITestsSignInfo = "Use Signing Info for signing the test server.";
            Arguments.UITestsToolsDir = "Path to the directory containing the Xamarin UI Tests tools including test-cloud.exe";
            Arguments.StopTestRunId = "ID of the started test run";
            Arguments.StatusTestRunId = "ID of the started test run";
            Arguments.StatusContinuous = "Continuously checks the test run status until it completes";
            Arguments.XCUITestIpaPath = "Path to the *.ipa file with the XCUITest tests";
            Arguments.XCUITestBuildDir = "Path to the build output directory (usually <project>/Build/Products/Debug-iphoneos)";
            Arguments.VSTSIdVariable = "Outputs the Test Run Id to the named VSTS variable";
            Arguments.TestOutputDir = "Path to the directory for test results. This parameter won't work when you run using --async because we have to wait for test results. The results will be saved in the working directory if a relative path is specified";
            Arguments.MergeNUnitXml = "Merge the xml files into the <arg> file. Requires --test-output-dir to specify the directory where the <arg> file will be. This parameter is ignored when used with the --async option because we have to wait for test results.";
            Arguments.MergeJUnitXml = "Merge the xml files into the <arg> file. Requires --test-output-dir to specify the directory where the <arg> file will be. This parameter is ignored when used with the --async option because we have to wait for test results.";
            Arguments.DownloadTestRunId = "ID of the started test run";
            Arguments.DownloadTestOutputDir = "Directory to download the .zip file(s) into for merging";
            Arguments.MergedFileName = "Name of the merged XML file";
            Arguments.Continuous = "Continuously checks the test run status until it completes";
            Arguments.NotSupported = "Attribute is not supported";
        })(Arguments = TestCloud.Arguments || (TestCloud.Arguments = {}));
    })(TestCloud = Messages.TestCloud || (Messages.TestCloud = {}));
})(Messages = exports.Messages || (exports.Messages = {}));
