/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Details of an uploaded release
 *
 */
class PrivateReleaseDetailsResponse {
  /**
   * Create a PrivateReleaseDetailsResponse.
   * @property {number} [id] ID identifying this unique release.
   * @property {string} [status] OBSOLETE. Will be removed in next version. The
   * availability concept is now replaced with distributed. Any 'available'
   * release will be associated with the default distribution group of an
   * app.</br>
   * The release state.<br>
   * <b>available</b>: The uploaded release has been distributed.<br>
   * <b>unavailable</b>: The uploaded release is not visible to the user. <br>
   * . Possible values include: 'available', 'unavailable'
   * @property {string} [appName] The app's name (extracted from the uploaded
   * release).
   * @property {string} [appDisplayName] The app's display name.
   * @property {string} [version] The release's version.<br>
   * For iOS: CFBundleVersion from info.plist.
   * For Android: android:versionCode from AppManifest.xml.
   * @property {string} [origin] The release's origin. Possible values include:
   * 'hockeyapp', 'appcenter'
   * @property {string} [shortVersion] The release's short version.<br>
   * For iOS: CFBundleShortVersionString from info.plist.
   * For Android: android:versionName from AppManifest.xml.
   * @property {string} [releaseNotes] The release's release notes.
   * @property {string} [provisioningProfileName] The release's provisioning
   * profile name.
   * @property {string} [provisioningProfileType] The type of the provisioning
   * profile for the requested app version. Possible values include: 'adhoc',
   * 'enterprise', 'other'
   * @property {boolean} [isProvisioningProfileSyncing] A flag that determines
   * whether the release's provisioning profile is still extracted or not.
   * @property {number} [size] The release's size in bytes.
   * @property {string} [minOs] The release's minimum required operating
   * system.
   * @property {string} [deviceFamily] The release's device family.
   * @property {string} [androidMinApiLevel] The release's minimum required
   * Android API level.
   * @property {string} [bundleIdentifier] The identifier of the apps bundle.
   * @property {string} [fingerprint] MD5 checksum of the release binary.
   * @property {string} [uploadedAt] UTC time in ISO 8601 format of the
   * uploaded time.
   * @property {string} [downloadUrl] The URL that hosts the binary for this
   * release.
   * @property {string} [appIconUrl] A URL to the app's icon.
   * @property {string} [installUrl] The href required to install a release on
   * a mobile device. On iOS devices will be prefixed with
   * `itms-services://?action=download-manifest&url=`
   * @property {string} [distributionGroupId] the destination where release is
   * distributed
   * @property {string} [publishingStatus] the publishing status of the
   * distributed release
   * @property {string} [destinationType] The destination type.<br>
   * <b>group</b>: The release distributed to internal groups and
   * distribution_groups details will be returned.<br>
   * <b>store</b>: The release distributed to external stores and
   * distribution_stores details will be returned. <br>
   * . Possible values include: 'group', 'store', 'tester'
   * @property {boolean} [isExternalBuild] This value determines if a release
   * is external or not.
   */
  constructor() {
  }

  /**
   * Defines the metadata of PrivateReleaseDetailsResponse
   *
   * @returns {object} metadata of PrivateReleaseDetailsResponse
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'PrivateReleaseDetailsResponse',
      type: {
        name: 'Composite',
        className: 'PrivateReleaseDetailsResponse',
        modelProperties: {
          id: {
            required: false,
            serializedName: 'id',
            type: {
              name: 'Number'
            }
          },
          status: {
            required: false,
            serializedName: 'status',
            type: {
              name: 'String'
            }
          },
          appName: {
            required: false,
            serializedName: 'app_name',
            type: {
              name: 'String'
            }
          },
          appDisplayName: {
            required: false,
            serializedName: 'app_display_name',
            type: {
              name: 'String'
            }
          },
          version: {
            required: false,
            serializedName: 'version',
            type: {
              name: 'String'
            }
          },
          origin: {
            required: false,
            serializedName: 'origin',
            type: {
              name: 'String'
            }
          },
          shortVersion: {
            required: false,
            serializedName: 'short_version',
            type: {
              name: 'String'
            }
          },
          releaseNotes: {
            required: false,
            serializedName: 'release_notes',
            type: {
              name: 'String'
            }
          },
          provisioningProfileName: {
            required: false,
            serializedName: 'provisioning_profile_name',
            type: {
              name: 'String'
            }
          },
          provisioningProfileType: {
            required: false,
            serializedName: 'provisioning_profile_type',
            type: {
              name: 'String'
            }
          },
          isProvisioningProfileSyncing: {
            required: false,
            serializedName: 'is_provisioning_profile_syncing',
            type: {
              name: 'Boolean'
            }
          },
          size: {
            required: false,
            serializedName: 'size',
            type: {
              name: 'Number'
            }
          },
          minOs: {
            required: false,
            serializedName: 'min_os',
            type: {
              name: 'String'
            }
          },
          deviceFamily: {
            required: false,
            serializedName: 'device_family',
            type: {
              name: 'String'
            }
          },
          androidMinApiLevel: {
            required: false,
            serializedName: 'android_min_api_level',
            type: {
              name: 'String'
            }
          },
          bundleIdentifier: {
            required: false,
            serializedName: 'bundle_identifier',
            type: {
              name: 'String'
            }
          },
          fingerprint: {
            required: false,
            serializedName: 'fingerprint',
            type: {
              name: 'String'
            }
          },
          uploadedAt: {
            required: false,
            serializedName: 'uploaded_at',
            type: {
              name: 'String'
            }
          },
          downloadUrl: {
            required: false,
            serializedName: 'download_url',
            type: {
              name: 'String'
            }
          },
          appIconUrl: {
            required: false,
            serializedName: 'app_icon_url',
            type: {
              name: 'String'
            }
          },
          installUrl: {
            required: false,
            serializedName: 'install_url',
            type: {
              name: 'String'
            }
          },
          distributionGroupId: {
            required: false,
            serializedName: 'distribution_group_id',
            type: {
              name: 'String'
            }
          },
          publishingStatus: {
            required: false,
            serializedName: 'publishing_status',
            type: {
              name: 'String'
            }
          },
          destinationType: {
            required: false,
            serializedName: 'destination_type',
            type: {
              name: 'String'
            }
          },
          isExternalBuild: {
            required: false,
            serializedName: 'is_external_build',
            type: {
              name: 'Boolean'
            }
          }
        }
      }
    };
  }
}

module.exports = PrivateReleaseDetailsResponse;
