/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * A request containing information for creating a release.
 *
 */
class ReleaseCreateRequest {
  /**
   * Create a ReleaseCreateRequest.
   * @property {string} [version] The release's version.<br>
   * For iOS: CFBundleVersion from info.plist.<br>
   * For Android: android:versionCode from AppManifest.xml.
   * @property {string} [buildVersion] The release's short version.<br>
   * For iOS: CFBundleShortVersionString from info.plist.<br>
   * For Android: android:versionName from AppManifest.xml.
   * @property {string} [uniqueIdentifier] The identifier of the app's bundle.
   * @property {string} [minimumOsVersion] The release's minimum required
   * operating system.
   * @property {string} [deviceFamily] The release's device family.
   * @property {array} [languages] The languages supported by the release.
   * Limited to 510 characters in a serialized array.
   * @property {string} fingerprint MD5 checksum of the release binary.
   * @property {number} size The release's size in bytes.
   * @property {string} [packageUrl] The URL to the release's binary.
   * @property {string} [fileExtension] The file extension of the asset. Does
   * not include the initial period.
   * @property {string} uploadId The upload id associated with the release, to
   * map to the releases upload table.
   * @property {string} [iconAssetId] The assetId associated with the icon
   * uploaded to app center file upload service.
   * @property {array} [ipaUuids] A list of UUIDs for architectures for an iOS
   * app.
   * @property {object} [provision] An object containing information about an
   * iOS provisioning profile.
   * @property {string} [provision.name] The name of the provisioning profile.
   * @property {string} [provision.applicationIdentifier] The application
   * identifier.
   * @property {string} [provision.teamIdentifier] The team identifier.
   * @property {string} [provision.profileType] Possible values include:
   * 'adhoc', 'enterprise', 'other'
   * @property {date} [provision.expiredAt] The profile's expiration date in
   * RFC 3339 format, i.e. 2017-07-21T17:32:28Z
   * @property {array} [provision.udids]
   * @property {array} [appexProvisioningProfiles] iOS app extension
   * provisioning profiles included in the release.
   * @property {boolean} [proxyFlow] If true this release was uploaded to the
   * AKS upload proxy
   */
  constructor() {
  }

  /**
   * Defines the metadata of ReleaseCreateRequest
   *
   * @returns {object} metadata of ReleaseCreateRequest
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ReleaseCreateRequest',
      type: {
        name: 'Composite',
        className: 'ReleaseCreateRequest',
        modelProperties: {
          version: {
            required: false,
            serializedName: 'version',
            type: {
              name: 'String'
            }
          },
          buildVersion: {
            required: false,
            serializedName: 'build_version',
            type: {
              name: 'String'
            }
          },
          uniqueIdentifier: {
            required: false,
            serializedName: 'unique_identifier',
            type: {
              name: 'String'
            }
          },
          minimumOsVersion: {
            required: false,
            serializedName: 'minimum_os_version',
            type: {
              name: 'String'
            }
          },
          deviceFamily: {
            required: false,
            serializedName: 'device_family',
            type: {
              name: 'String'
            }
          },
          languages: {
            required: false,
            serializedName: 'languages',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          fingerprint: {
            required: true,
            serializedName: 'fingerprint',
            type: {
              name: 'String'
            }
          },
          size: {
            required: true,
            serializedName: 'size',
            type: {
              name: 'Number'
            }
          },
          packageUrl: {
            required: false,
            serializedName: 'package_url',
            type: {
              name: 'String'
            }
          },
          fileExtension: {
            required: false,
            serializedName: 'file_extension',
            type: {
              name: 'String'
            }
          },
          uploadId: {
            required: true,
            serializedName: 'upload_id',
            type: {
              name: 'String'
            }
          },
          iconAssetId: {
            required: false,
            serializedName: 'icon_asset_id',
            type: {
              name: 'String'
            }
          },
          ipaUuids: {
            required: false,
            serializedName: 'ipa_uuids',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ReleaseCreateRequestIpaUuidsItemElementType',
                  type: {
                    name: 'Composite',
                    className: 'ReleaseCreateRequestIpaUuidsItem'
                  }
              }
            }
          },
          provision: {
            required: false,
            serializedName: 'provision',
            type: {
              name: 'Composite',
              className: 'ReleaseCreateRequestProvision'
            }
          },
          appexProvisioningProfiles: {
            required: false,
            serializedName: 'appex_provisioning_profiles',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ReleaseCreateRequestAppexProvisioningProfilesItemElementType',
                  type: {
                    name: 'Composite',
                    className: 'ReleaseCreateRequestAppexProvisioningProfilesItem'
                  }
              }
            }
          },
          proxyFlow: {
            required: false,
            serializedName: 'proxy_flow',
            type: {
              name: 'Boolean'
            }
          }
        }
      }
    };
  }
}

module.exports = ReleaseCreateRequest;
